package gregapi.render;

import static gregapi.data.CS.UNCOLOURED;
import gregapi.util.UT;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.util.IIcon;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;

/** 
 * @author Gregorius Techneticies
 */
public class BlockTextureFluid implements ITexture {
	private final FluidStack mFluidStack;
	private final Fluid mFluid;
	private final int mLuminosity;
	
	private final boolean mAllowAlpha;
	
	/**
	 *  DO NOT MANIPULATE THE VALUES INSIDE THIS ARRAY!!!
	 *  
	 *  Just set this variable to another different Array instead.
	 *  Otherwise some coloured things will get Problems.
	 */
	public short[] mRGBa;
	
	public BlockTextureFluid(FluidStack aFluidStack, boolean aAllowAlpha) {
		if (aFluidStack == null) {
			mFluidStack = null;
			mFluid = null;
			mLuminosity = 0;
			mRGBa = UNCOLOURED;
		} else {
			mFluidStack = aFluidStack;
			mFluid = mFluidStack.getFluid();
			mLuminosity = mFluid.getLuminosity(mFluidStack) * 16;
			mRGBa = UT.Code.getRGBaArray(mFluid.getColor(mFluidStack));
		}
		mAllowAlpha = aAllowAlpha;
	}
	
	public BlockTextureFluid(FluidStack aFluidStack) {
		this(aFluidStack, false);
	}
	
	public BlockTextureFluid(Fluid aFluid, boolean aAllowAlpha) {
		this(new FluidStack(aFluid, 0), aAllowAlpha);
	}
	
	public BlockTextureFluid(Fluid aFluid) {
		this(aFluid, false);
	}
	
	private IIcon getIcon(int aSide) {
		return mFluid.getIcon(mFluidStack);
	}
	
	@Override
	public void renderXPos(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness) {
//		aRenderer.flipTexture = !aRenderer.flipTexture;
    	ITexture.Util.renderXPos(getIcon(5), mRGBa, mAllowAlpha, mLuminosity > aBrightness, true, aRenderer, aBlock, aX, aY, aZ, Math.max(mLuminosity, aBrightness));
//		aRenderer.flipTexture = !aRenderer.flipTexture;
	}
	
	@Override
	public void renderXNeg(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness) {
    	ITexture.Util.renderXNeg(getIcon(4), mRGBa, mAllowAlpha, mLuminosity > aBrightness, true, aRenderer, aBlock, aX, aY, aZ, Math.max(mLuminosity, aBrightness));
	}
	
	@Override
	public void renderYPos(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness) {
    	ITexture.Util.renderYPos(getIcon(1), mRGBa, mAllowAlpha, mLuminosity > aBrightness, true, aRenderer, aBlock, aX, aY, aZ, Math.max(mLuminosity, aBrightness));
	}
	
	@Override
	public void renderYNeg(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness) {
    	ITexture.Util.renderYNeg(getIcon(0), mRGBa, mAllowAlpha, mLuminosity > aBrightness, true, aRenderer, aBlock, aX, aY, aZ, Math.max(mLuminosity, aBrightness));
	}
	
	@Override
	public void renderZPos(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness) {
    	ITexture.Util.renderZPos(getIcon(3), mRGBa, mAllowAlpha, mLuminosity > aBrightness, true, aRenderer, aBlock, aX, aY, aZ, Math.max(mLuminosity, aBrightness));
	}
	
	@Override
	public void renderZNeg(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness) {
//		aRenderer.flipTexture = !aRenderer.flipTexture;
		ITexture.Util.renderZNeg(getIcon(2), mRGBa, mAllowAlpha, mLuminosity > aBrightness, true, aRenderer, aBlock, aX, aY, aZ, Math.max(mLuminosity, aBrightness));
//		aRenderer.flipTexture = !aRenderer.flipTexture;
	}
	
	@Override
	public boolean isValidTexture() {
		return mFluid != null;
	}
}