package gregapi.util;

import static gregapi.data.CS.U;
import gregapi.data.MT;
import gregapi.data.OP;
import gregapi.oredict.*;

import java.util.List;

import net.minecraft.item.ItemStack;

/**
 * @author Gregorius Techneticies
 * 
 * Utility for accessing the OreDictUnificator Data in a more short manner. The Short Name is for ease of overview and stands for "OredictManager".
 */
public class OM {
	public static ItemStack set(ItemStack aStack) {
		return OreDictManager.INSTANCE.setStack(true, aStack);
	}
	
	public static ItemStack get(ItemStack aStack) {
		return OreDictManager.INSTANCE.getStack(true, aStack);
	}
	
	public static ItemStack get(OreDictPrefix aPrefix, OreDictMaterial aMaterial, long aStackSize) {
		return OreDictManager.INSTANCE.getStack(aPrefix, aMaterial, aStackSize);
	}
	
	public static ItemStack get(OreDictPrefix aPrefix, OreDictMaterial aMaterial, ItemStack aReplacement, long aStackSize) {
		return OreDictManager.INSTANCE.getStack(aPrefix, aMaterial, aReplacement, aStackSize);
	}
	
	public static void data(ItemStack aStack, OreDictItemData aData) {
		OreDictManager.INSTANCE.addItemData(aStack, aData);
	}
	
	public static OreDictItemData data(ItemStack aStack) {
		return OreDictManager.INSTANCE.getItemData(aStack);
	}
	
	public static void association(ItemStack aStack, OreDictPrefix aPrefix, OreDictMaterial aMaterial) {
		OreDictManager.INSTANCE.addAssociation(aPrefix, aMaterial, aStack);
	}
	
	public static OreDictItemData association(ItemStack aStack) {
		return OreDictManager.INSTANCE.getAssociation(aStack);
	}
	
	public static void blacklist(ItemStack aStack) {
		OreDictManager.INSTANCE.addToBlacklist(aStack);
	}
	
	public static long total(List<OreDictMaterialStack> aList) {
		long rAmount = 0;
		for (OreDictMaterialStack tStack : aList) if (tStack != null && tStack.mMaterial != MT.NULL) rAmount += tStack.mAmount;
		return rAmount;
	}
	
	public static double weight(List<OreDictMaterialStack> aList) {
		double rWeight = 0;
		for (OreDictMaterialStack tStack : aList) if (tStack != null && tStack.mMaterial != MT.NULL) rWeight += tStack.mMaterial.getWeight(tStack.mAmount);
		return rWeight;
	}
	
    public static ItemStack gem(OreDictMaterialStack aMaterial) {
    	return aMaterial==null?null:gem(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack gem(OreDictMaterial aMaterial, OreDictPrefix aPrefix) {
    	return aMaterial==null?null:gem(aMaterial, aPrefix.mAmount);
    }
    
    public static ItemStack gem(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = null;
    	if (																							   aMaterialAmount >= U * 72 ) rStack = get(OP.blockGem		, aMaterial,  aMaterialAmount      / (U * 9));
    	if (rStack == null && (( aMaterialAmount      %   U     == 0)									|| aMaterialAmount >= U * 32)) rStack = get(OP.gem			, aMaterial,  aMaterialAmount      /  U     );
    	if (rStack == null && (((aMaterialAmount * 2) %   U     == 0)									|| aMaterialAmount >= U * 16)) rStack = get(OP.gemFlawed	, aMaterial, (aMaterialAmount * 2) /  U     );
    	if (rStack == null && (((aMaterialAmount * 4) >=  U         )                             									)) rStack = get(OP.gemChipped	, aMaterial, (aMaterialAmount * 4) /  U     );
    	return rStack;
    }
    
    public static ItemStack dust(OreDictMaterialStack aMaterial) {
    	return aMaterial==null?null:dust(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack dust(OreDictMaterial aMaterial, OreDictPrefix aPrefix) {
    	return aMaterial==null?null:dust(aMaterial, aPrefix.mAmount);
    }
    
    public static ItemStack dust(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = null;
    	if (																							   aMaterialAmount >= U * 72 ) rStack = get(OP.blockDust	, aMaterial,  aMaterialAmount      / (U * 9));
    	if (rStack == null && (( aMaterialAmount      %   U     == 0)									|| aMaterialAmount >= U * 16)) rStack = get(OP.dust			, aMaterial,  aMaterialAmount      /  U     );
		if (rStack == null && (((aMaterialAmount * 4) %   U     == 0)									|| aMaterialAmount >= U *  8)) rStack = get(OP.dustSmall	, aMaterial, (aMaterialAmount * 4) /  U     );
		if (rStack == null && (((aMaterialAmount * 9) >=  U         )                             									)) rStack = get(OP.dustTiny		, aMaterial, (aMaterialAmount * 9) /  U     );
    	return rStack;
    }
    
    public static ItemStack ingot(OreDictMaterialStack aMaterial) {
    	return aMaterial==null?null:ingot(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack ingot(OreDictMaterial aMaterial, OreDictPrefix aPrefix) {
    	return aMaterial==null?null:ingot(aMaterial, aPrefix.mAmount);
    }
    
    public static ItemStack ingot(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = null;
    	if (                  																			   aMaterialAmount >= U * 72 ) rStack = get(OP.blockIngot	, aMaterial,  aMaterialAmount      / (U * 9));
    	if (rStack == null && (( aMaterialAmount      %   U     == 0                                  ) || aMaterialAmount >= U * 16)) rStack = get(OP.ingot		, aMaterial,  aMaterialAmount      /  U     );
    	if (rStack == null && (((aMaterialAmount * 4) %   U     == 0)									|| aMaterialAmount >= U *  8)) rStack = get(OP.chunkGt		, aMaterial, (aMaterialAmount * 4) /  U     );
		if (rStack == null && (((aMaterialAmount * 9) >=  U                                           )                             )) rStack = get(OP.nugget		, aMaterial, (aMaterialAmount * 9) /  U     );
    	return rStack;
    }
    
    public static ItemStack solid(OreDictMaterialStack aMaterial) {
    	return aMaterial==null?null:solid(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack solid(OreDictMaterial aMaterial, OreDictPrefix aPrefix) {
    	return aMaterial==null?null:solid(aMaterial, aPrefix.mAmount);
    }
    
    public static ItemStack solid(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = null;
    	if (                  																			   aMaterialAmount >= U * 72 ) rStack = get(OP.blockSolid	, aMaterial,  aMaterialAmount      / (U * 9));
    	if (rStack == null && (( aMaterialAmount      %   U     == 0                                  ) || aMaterialAmount >= U * 16)) rStack = get(OP.ingot		, aMaterial,  aMaterialAmount      /  U     );
    	if (rStack == null && (((aMaterialAmount * 4) %   U     == 0)									|| aMaterialAmount >= U *  8)) rStack = get(OP.chunkGt		, aMaterial, (aMaterialAmount * 4) /  U     );
		if (rStack == null && (((aMaterialAmount * 9) >=  U                                           )                             )) rStack = get(OP.nugget		, aMaterial, (aMaterialAmount * 9) /  U     );
    	return rStack;
    }
    
    public static ItemStack ingotOrDust(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = ingot(aMaterial, aMaterialAmount);
    	if (rStack == null) rStack = dust(aMaterial, aMaterialAmount);
    	return rStack;
    }
    
    public static ItemStack ingotOrDust(OreDictMaterialStack aMaterial) {
    	return aMaterial == null ? null : ingotOrDust(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack dustOrIngot(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = dust(aMaterial, aMaterialAmount);
    	return rStack == null ? ingot(aMaterial, aMaterialAmount) : rStack;
    }
    
    public static ItemStack dustOrIngot(OreDictMaterialStack aMaterial) {
    	return aMaterial == null ? null : dustOrIngot(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack solidOrDust(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = solid(aMaterial, aMaterialAmount);
    	return rStack == null ? dust(aMaterial, aMaterialAmount) : rStack;
    }
    
    public static ItemStack solidOrDust(OreDictMaterialStack aMaterial) {
    	return aMaterial == null ? null : solidOrDust(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
    public static ItemStack dustOrSolid(OreDictMaterial aMaterial, long aMaterialAmount) {
    	if (aMaterialAmount <= 0 || aMaterial == null) return null;
    	ItemStack rStack = dust(aMaterial, aMaterialAmount);
    	return rStack == null ? solid(aMaterial, aMaterialAmount) : rStack;
    }
    
    public static ItemStack dustOrSolid(OreDictMaterialStack aMaterial) {
    	return aMaterial == null ? null : dustOrSolid(aMaterial.mMaterial, aMaterial.mAmount);
    }
    
	public static OreDictMaterialStack stack(OreDictMaterial aMaterial, long aAmount) {return aMaterial==null?null:new OreDictMaterialStack(aMaterial, aAmount);}
	public static OreDictMaterialStack stack(long aAmount, OreDictMaterial aMaterial) {return aMaterial==null?null:new OreDictMaterialStack(aMaterial, aAmount);}
	public static OreDictMaterialStack stack(OreDictMaterial aMaterial, OreDictPrefix aPrefix) {return aMaterial==null||aPrefix==null?null:new OreDictMaterialStack(aMaterial, aPrefix.mAmount);}
	public static OreDictMaterialStack stack(OreDictPrefix aPrefix, OreDictMaterial aMaterial) {return aMaterial==null||aPrefix==null?null:new OreDictMaterialStack(aMaterial, aPrefix.mAmount);}
}